;**************************************************************************
;*
;* FILE mlboot.asm
;* Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: mlboot.asm,v 1.3 2006-09-08 09:52:37 tlr Exp $
;*
;* DESCRIPTION
;*   The ml-version of the boot protocol.
;*
;*   SYS53000
;*
;******
	processor 6502

chk_zp	equ	$fb
dest_zp	equ	$ac
tmp_zp	equ	$fc

	ifnconst	AUTO
PORT_MODE_JOY1	  equ	1
	endif
	
	ifconst	PORT_MODE_JOY1
PORT		equ	$dc01
PORT_BITS	equ	%00011111
PORT_DDR	equ	$dc03
PORT_DDR_DEF	equ	%00000000
PORT_3_7	equ	0
	endif
	ifconst	PORT_MODE_JOY2
PORT		equ	$dc00
PORT_BITS	equ	%00011111
PORT_DDR	equ	$dc02
PORT_DDR_DEF	equ	%11111111
PORT_3_7	equ	0
	endif
	ifconst	PORT_MODE_USER0_4
PORT		equ	$dd01
PORT_BITS	equ	%00011111
PORT_DDR	equ	$dd03
PORT_DDR_DEF	equ	%00000000
PORT_3_7	equ	0
	endif
	ifconst	PORT_MODE_USER3_7
PORT		equ	$dd01
PORT_BITS	equ	%11111000
PORT_DDR	equ	$dd03
PORT_DDR_DEF	equ	%00000000
PORT_3_7	equ	1
	endif
	
	seg	code
	org	$cf08		;=53000
;**************************************************************************
;*
;* NAME  main
;*
;* DESCRIPTION
;*   Entry Point.
;*
;******
main:
	if 0
;*   SYS53000,56321,1  - receive on Joy port 1
;*   SYS53000,56320,1  - receive on Joy port 2
;*   SYS53000,56577,1  - receive on Userport4-0
;*   SYS53000,56577,8  - receive on Userport7-3
	jsr	$0079	;CHRGOT
	beq	mn_skp3
	jsr	$aefd	;Check for ','
	jsr	$b7eb	;Get parameters for poke ($14/$15),X
	inc	$d021
mn_skp3:
	endif
	
	sei
; Preserve port configuration
	lda	PORT_DDR
	pha
	lda	PORT
	pha
	
; JoyA0-JoyA3=input, JoyA4=output
	lda	#%00010000
	sta	PORT_DDR
; all ones (tri-state)
	lda	#%11111111
	sta	PORT

	lda	#0
	sta	$d020
	
	jsr	getbyte
	sta	dest_zp+1
	jsr	getbyte
	sta	dest_zp
	jsr	getbyte
	sta	$af
	jsr	getbyte
	clc
	adc	#1
	sta	$ae
	sta	$2d
	lda	$af
	adc	#0
	sta	$af
	sta	$2e
	
	ldy	#0
	sty	chk_zp
mn_lp1:
	jsr	getbyte
	sta	$d020
	sta	(dest_zp),y
	sec
	adc	chk_zp
	sta	chk_zp
	inc	dest_zp
	bne	mn_skp1
	inc	dest_zp+1
mn_skp1:
	lda	dest_zp
	cmp	$ae
	bne	mn_lp1
	lda	dest_zp+1
	cmp	$af
	bne	mn_lp1
	
; get checksum.  Set color=red if fail, and color=green if ok.
	ldy	#2
	jsr	getbyte
	cmp	chk_zp
	bne	mn_skp2
	ldy	#5
mn_skp2:
	sty	$d020

; wait for all port bits to go high before exiting
mn_lp2:	
	lda	PORT
	and	#PORT_BITS
	cmp	#PORT_BITS
	bne	mn_lp2

; Restore port configuration
	pla
	sta	PORT
	pla
	sta	PORT_DDR
; exit
	cli
	jmp     $a474   ;Exit to BASIC DIRECTMODE
	
;**************************************************************************
;*
;* NAME  getbyte
;*
;* DESCRIPTION
;*   Get a byte from the master.
;*
;******
getbyte:
	ldx	#2
gb_lp1:	
	lda	#%00001000
; Wait for CLK low
gb_lp2:
	bit	PORT
	bne	gb_lp2
; D1/D0 contains databits 7/6 or 3/2
	jsr	get2bit
	
	lda	#%00001000
; Wait for CLK high
gb_lp3:
	bit	PORT
	beq	gb_lp3
; D1/D0 contains databits 5/4 or 1/0
	jsr	get2bit
	
	dex
	bne	gb_lp1
	lda	tmp_zp
	rts

get2bit:
 	lda	PORT		
; ACK=CLK by shifting CLK into ACK.
 	asl	PORT
	if	PORT_3_7
	asl
	asl
	rol	tmp_zp
	asl
	rol	tmp_zp
	else
	and	#%00000011
	asl	tmp_zp
	asl	tmp_zp
	ora	tmp_zp
	sta	tmp_zp
	endif
	rts
; eof
