/*************************************************************************
 *
 * FILE  membuffer.cpp
 * Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: membuffer.cpp,v 1.6 2006-11-29 20:10:08 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/

#include <iostream>
#include <fstream>
#include <string>
#include "util/types.hpp"
#include "membuffer.hpp"

void Address::set_start(const char* str)
{
    char* endptr;

    rom_=false;
    if (*str=='r' || *str=='R') {
	str++;
	rom_=true;
    }
    uint32_t start=std::strtoul(str,&endptr,0);
    start &=0xffffff;
    if (endptr==str || *endptr!=0) {
	// failed
	return;
    }
    start_ = start;
}


void AddressRange::set_range(const char* str)
{
    char* endptr;

    if (*str=='r' || *str=='R') {
	str++;
	rom_=true;
    }
    uint32_t start=std::strtoul(str,&endptr,0);
    start &=0xffffff;
    if (endptr==str) {
	// failed
	return;
    }
    str=endptr;
    char sep=*str++;
    if ( !(sep == '-' || sep == ',') ) {
	// failed
	return;
    }
    uint32_t end=std::strtoul(str,&endptr,0);
    end &=0xffffff;
    if (endptr==str) {
	// failed
	return;
    }
    /* parsed, set. */

    if (sep=='-') {
	start_=start;
	length_=end-start;
    }
    if (sep==',') {
	start_=start;
	length_=end;
    }
}

void MemBuffer::read_file(const char* name, bool use_prg)
{
    std::ifstream is;
    is.open(name, std::ios::binary);
    is.seekg(0, std::ios::end);
    uint32_t len=is.tellg();
    is.seekg(0, std::ios::beg);

    uint32_t sa;
    
    if (use_prg) {
	char c;
	is.get(c);
	sa=static_cast<uint8_t>(c);
	is.get(c);
	sa|=static_cast<uint8_t>(c)<<8;
	len-=2;
    }

    delete buffer_;
    buffer_=new uint8_t[len];

    for (unsigned int i=0; i < len; ++i) {
	char c;
	is.get(c);
	buffer_[i]=static_cast<uint8_t>(c);
    }

    length_=len;
    start_=sa;

    is.close();
}

void MemBuffer::write_file(const char* name, bool use_prg) const
{
    std::ofstream os(name, std::ios::binary);

    if (use_prg) {
	uint32_t sa = get_start();
	os.put( sa & 0xff );
	os.put( (sa >> 8) & 0xff );
    }

    for (unsigned int i=0; i < get_length(); ++i)
	os.put(buffer_[i]);

    os.close();
}
/* eof */
